/*

  Create a fits image based on a given model and dataset.

  Requires the file "chomp.h" and <funtools.h> plus the standard headers to have been previously declared

*/

//Definitions
#ifndef MAXCMDLENGTH
#define MAXCMDLENGTH 1024
#endif

#ifndef MAXNUMFILES
#define MAXNUMFILES 50
#endif

#ifndef GFACTOR
#define GFACTOR (2.0*sqrt(2.0*log(2.0)))
#endif


// Declare the function prototypes

int syscom(const char[]);

int findmodelvalues(double usr_gmin, double usr_gmax, double fieldstrength, int model, float *reconmapflux, int xdim, int ydim, float usr_frequency, float *bestnorm, float *bestage, int **regionarray, float inject, float beamarea, float redshift);

int mapfrommodel(char *imageloc, char *dirname, int xdim, int ydim, char *maptarget, double usr_gmin, double usr_gmax, double fieldstrength, int model, float *bestnorm, float *bestage, int **regionarray, float inject, float beamarea, float redshift) { 


  // Declare the variables for this task

  char **fitsfiles, modelname[32];

  int required, i;
  float *reconmapflux;
  char *fileloc, *reconnloc, imagename[256];

  FITSHead head;
  Fun *fitsconn, *modelconn;
  DIR *direct;

  float usr_frequency = 1e9;

  struct dirent *entry;

  // Get the directory name and format it
 
  required = 1;

  while (required == 1) {

    // Get the scaling factor
    printf("Please enter a frequency (in Hz) for which to construct the image (-1 to escape): ");

    if (scanf("%f", &usr_frequency) == 0) {
		 
      printf("Invalid input, escaping command to protect the existing data...\n");
      return(7);
    }
    else if (usr_frequency < 0.0) {
      printf("Escaping command...\n\n");
      return(100);
    }
    else {
      printf("Combined map set to a frequency of %.2e Hz\n", usr_frequency);
      required = 0;
    }
  }


  /* Check all the files and directories exists then put them in the correct format  */

  // Allocating memory

  fitsfiles = (char **)calloc(MAXNUMFILES, sizeof(char *)); 
  
  //Open the directory
  direct = opendir(dirname);

  //Set the map counter to 0
  i = 0;

  //Check the directory exists
  if (direct != NULL) {

    // While there are maps to read...
    while ((entry = readdir(direct))) {
      if (strlen(entry->d_name) > MAXCMDLENGTH+1) {
	fprintf(stderr,"\n*** Error: File path must be less than %d characters long! ***\n\n", MAXCMDLENGTH);
	return 2;
      }
      //Do nothing if we are reading . or ..
      else if ((strcmp(entry->d_name, ".") == 0) || (strcmp(entry->d_name, "..") == 0)) {
	;
      }

      // Get the fits file data...
      else {
	fitsfiles[i] = (char *)calloc((MAXCMDLENGTH+1),sizeof(char));
	strcpy(fitsfiles[i], entry->d_name);

	//Increase the map counter
	i++;
      }
    }

    closedir (direct);
  }

  //If we can't find or access the directory, kick back to the command prompt
  else {
    fprintf(stderr,"*** Error: Unable to locate FITS directory ***\n\n");
    return 404;
  }

  /* Go off and get the data */

  printf("Determining the reconstructed map fluxes...\n");

  reconmapflux = (float *)calloc(xdim*ydim, sizeof(float));

  findmodelvalues(usr_gmin, usr_gmax, fieldstrength, model, reconmapflux, xdim, ydim, usr_frequency, bestnorm, bestage, regionarray, inject, beamarea, redshift);


  fileloc = (char *)calloc(MAXCMDLENGTH,sizeof(char));
  reconnloc = (char *)calloc(MAXCMDLENGTH,sizeof(char));

  printf("Loading (real) map parameters...\n");

   // Create the map

  if (model == 1) {
    sprintf(modelname,"JP");
  }
  else if (model == 2) {
    sprintf(modelname,"KP");
  }
  else if (model == 3) {
    sprintf(modelname,"JPTRIB");
  }
  else if (model == 4) {
    sprintf(modelname,"KPTRIB");
  }
  else {
    sprintf(modelname,"Unknown");
  }

  sprintf(imagename,"%s_%.2e_%s_recon.fits", maptarget, usr_frequency, modelname);
  sprintf(reconnloc,"./%s/%s", imageloc, imagename);

   // Create the file and write the header (currently AIPS format)
    
    modelconn = FunOpen(reconnloc, "w", NULL);

    // Check we can access the file
    if (modelconn == NULL){
      fprintf(stderr,"\n*** Error: Unable to open %s, please check the file exists and access permissions are correct. ***\n\n", fitsfiles[i]);
      return 404;
    }
    
    // Get the header of the first map loaded
    sprintf(fileloc,"./%s/%s", dirname, fitsfiles[0]);
    fitsconn = FunOpen(fileloc, "r", NULL);

    // Check we can access the file
    if (fitsconn == NULL){
      fprintf(stderr,"\n*** Error: Unable to open %s, please check the file exists and access permissions are correct. ***\n\n", fitsfiles[i]);
      return 404;
    }

    FunInfoGet(fitsconn, FUN_HEADER, &head, 0);

    // Set this as the header for the new map
    FunInfoPut(modelconn, FUN_HEADER, &head, 0);

    // Set any parameters which have been changed.
    /*
    FunParamPuts(modelconn, "OBJECT", 0,  maptarget, "Target Source", 1);
    FunParamPuts(modelconn, "BUNIT", 0,  units[0], "Units", 1);
    FunParamPutd(modelconn, "CRVAL3", 0,  (double)usr_frequency, 11, "Map Frequency", 1);
    FunParamPutd(modelconn, "CDELT1", 0,  (double)delt2, 9, "", 1);
    FunParamPutd(modelconn, "CDELT2", 0,  (double)delt, 9, "", 1);
    */

    FunParamPutd(modelconn, "CRVAL3", 0,  (double)usr_frequency, 11, "Map Frequency", 1);

    // Output the image
    FunImagePut(modelconn, reconmapflux, xdim, ydim, -32, NULL);
    FunClose(modelconn);
    FunClose(fitsconn);


    printf("\nReconstructed map has been exported to %s\n\n", reconnloc);


  //Free up the memory
  for (i=0; i<MAXNUMFILES; i++) {
    free(fitsfiles[i]);
  }

  free(fileloc);
  free(fitsfiles);
  free(reconmapflux);

  return 0;

}
